;;_____________________________________________________________________________
;;-----------------------------------------------------------------------------
;; Emacs IDE for Drupal (http://drupal.org/project/emacs)
;; drupal-mode.el — major mode for Drupal coding
;; 
;; Under GPLv2 or above
;;-----------------------------------------------------------------------------

(require 'php-mode)

;;-----------------------------------------------------------------------------
;; Drupal mode
;;
(define-derived-mode drupal-mode php-mode "Drupal"
  "Major mode for Drupal coding.\n\n\\{drupal-mode-map}"
	:group 'drupal
	(setq tab-width 2) ; by Drupal coding standard
  (setq c-basic-offset 2)
  (setq indent-tabs-mode nil)
  (setq fill-column 78)
  (setq show-trailing-whitespace t)
  (add-hook 'before-save-hook 'delete-trailing-whitespace)
  (c-set-offset 'case-label '+)
  (c-set-offset 'arglist-close 0)
  (c-set-offset 'arglist-intro '+) ; for FAPI arrays and DBTNG
  (c-set-offset 'arglist-cont-nonempty 'c-lineup-math) ; for DBTNG fields and values
	(run-mode-hooks 'drupal-mode-hook)
)

;;-----------------------------------------------------------------------------
;; Helpers
;;
(defun drupal-module-name () ;; based on sacha chua's idea
  "Return the Drupal module name for .module and .install files."
  (file-name-sans-extension (file-name-nondirectory (buffer-file-name))))

;;-----------------------------------------------------------------------------
;; Interactive functions
;;
(defun drupal-module-name-insert ()
	 "Just insert Drupal module name at a point."
	 (interactive)
	 (insert (drupal-module-name)))

(defun drupal-browse-api () ; Based on hint by Sean Charles http://drupal.org/node/249296
	"Browse Drupal API docs."
	(interactive)
	(browse-url
	 (format "%s/%i/%s" drupal-api-url drupal-api-version (symbol-at-point))))

;;-----------------------------------------------------------------------------
;; Flymake realtime syntax checking
;;
(require 'flymake)

(defun flymake-php-init ()
  "Use php to check the syntax of the current file."
  (let* ((temp (flymake-init-create-temp-buffer-copy 'flymake-create-temp-inplace))
	 (local (file-relative-name temp (file-name-directory buffer-file-name))))
    (list "php" (list "-f" local "-l"))))

(add-to-list 'flymake-err-line-patterns
  '("\\(Parse\\|Fatal\\) error: +\\(.*?\\) in \\(.*?\\) on line \\([0-9]+\\)$" 3 4 nil 2))

(add-to-list 'flymake-allowed-file-name-masks '("\\.php$" flymake-php-init))
; + Drupal extensions
(add-to-list 'flymake-allowed-file-name-masks '("\\.module$" flymake-php-init))
(add-to-list 'flymake-allowed-file-name-masks '("\\.install$" flymake-php-init))
(add-to-list 'flymake-allowed-file-name-masks '("\\.inc$" flymake-php-init))
(add-to-list 'flymake-allowed-file-name-masks '("\\.engine$" flymake-php-init))

(add-hook 'drupal-mode-hook (lambda () (flymake-mode 1)))

;;-----------------------------------------------------------------------------
;; Yasnippet
;;
(if (require 'yasnippet "yasnippet" t) ; TODO test it
		(progn
			(add-to-list 'yas/root-directory (concat user-emacs-directory 
																							 (convert-standard-filename "snippets"))
									 (add-hook 'drupal-mode-hook (lambda ()
																								 (yas/initialize)
																								 (mapc 'yas/load-directory yas/root-directory)))))

;;-----------------------------------------------------------------------------
;; Keybindings
;;
(define-key php-mode-map '[M-S-up] 'flymake-goto-prev-error)
(define-key php-mode-map '[M-S-down] 'flymake-goto-next-error)
(define-key drupal-mode-map "\C-hf" 'drupal-browse-api)
(define-key drupal-mode-map "\C-cm" 'drupal-module-name-insert)

(provide 'drupal-mode)
